/* $Id: transcript.h 224437 2021-07-21 16:53:46Z twu $ */
#ifndef TRANSCRIPT_INCLUDED
#define TRANSCRIPT_INCLUDED
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "filestring.h"
#include "list.h"
#include "bool.h"
#include "genomicpos.h"
#include "iit-read-univ.h"
#include "outputtype.h"
#include "transcriptome.h"
#include "listpool.h"


/* Ordered for preference in choosing among hitpairs, but order for accept_velocity is different */
typedef enum {V_INVALID, V_UNSPLICED, V_EXTENDED, V_AMBIGUOUS, V_SPLICED} Transcript_velocity_T;

#define T Transcript_T
typedef struct T *T;

extern int Transcript_num (T this);
extern int Transcript_start (T this);
extern int Transcript_end (T this);
extern int Transcript_length (T this);
extern int Transcript_genestrand (T this);

extern void
Transcript_setup (int pairmax_transcriptome_in, int expected_pairlength_in,
		  Outputtype_T output_type_in, Transcriptome_T transcriptome_in,
		  Univ_IIT_T transcript_iit_in);

extern void
Transcript_free (T *old);
extern void
Transcript_gc (List_T *list);
extern T
Transcript_new_need_tokens (int num, int transcript_genestrand,
			    int trstart, int trend, int trlength,
			    int trim_low, int trim_high, Listpool_T listpool);
extern T
Transcript_new_with_tokens (int num, int transcript_genestrand,
			    int trstart, int trend, int trlength,
			    int querystart, int queryend, int querylength,
			    List_T tokens);

extern T
Transcript_copy (T old, Listpool_T listpool);
extern List_T
Transcript_copy_list (List_T old, Listpool_T listpool);
extern void
Transcript_trim (T this, int trim_low_adj, int trim_high_adj, Listpool_T listpool);

extern bool
Transcript_extend_utr5p (List_T transcripts);
extern bool
Transcript_extend_utr3p (List_T transcripts);

extern bool
Transcript_in_list_p (T x, List_T list);
extern void
Transcript_print_nums (List_T list);
extern void
Transcript_print_list_debug (List_T list);

/* Depends on pairmax_transcriptome */
extern bool
Transcript_concordant_p (int *desired_genestrand,
			 Transcript_velocity_T *search_velocity,
			 Transcript_velocity_T *accept_velocity,
			 List_T transcripts5, List_T transcripts3);

extern List_T
Transcript_get_concordant (int desired_genestrand,
			   Transcript_velocity_T search_velocity,
			   Transcript_velocity_T accept_velocity,
			   List_T transcripts5, List_T transcripts3,
			   Listpool_T listpool, bool first_read_p);

extern List_T
Transcript_get_singleend (int *transcript_genestrand, List_T transcripts,
			  Listpool_T listpool);

extern void
Transcript_print_list (Filestring_T fp, List_T transcripts,
		       Univ_IIT_T transcript_iit, bool invertp, char *header);

#undef T
#endif

